# Generated by Django 3.2.13 on 2022-06-28 13:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='xiaconfiguration',
            name='source_target_mapping',
        ),
        migrations.AddField(
            model_name='xiaconfiguration',
            name='xss_api',
            field=models.URLField(default='https://localhost', help_text='Enter the XSS API'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='metadatafieldoverwrite',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='source_metadata_schema',
            field=models.CharField(help_text='Enter the schema name/IRI', max_length=200),
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='target_metadata_schema',
            field=models.CharField(help_text='Enter the target schema name/IRI to validate from.', max_length=200),
        ),
        migrations.AlterField(
            model_name='xisconfiguration',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]
