# Generated by Django 3.2.18 on 2025-02-19 17:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0007_auto_20250219_1719'),
    ]

    operations = [
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='key_fields',
            field=models.TextField(default='["p2881_course_profile.Course_ID", "p2881_course_profile.CourseProviderName"]', help_text='Enter list of field names to create metadata key'),
        ),
    ]
