# Generated by Django 3.2.18 on 2025-02-25 15:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0008_alter_xiaconfiguration_key_fields'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='xiaconfiguration',
            name='source_file',
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='key_fields',
            field=models.TextField(blank=True, default='["p2881_course_profile.Course_ID", "p2881_course_profile.CourseProviderName"]', help_text='Enter list of field names to create metadata key', null=True),
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='source_metadata_schema',
            field=models.CharField(blank=True, help_text='Enter the schema name/IRI', max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='target_metadata_schema',
            field=models.CharField(blank=True, help_text='Enter the target schema name/IRI to validate from.', max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='xiaconfiguration',
            name='xss_api',
            field=models.CharField(blank=True, help_text='Enter the XSS API', max_length=200, null=True),
        ),
    ]
