#!/Users/kjijo/Library/CloudStorage/OneDrive-Deloitte(O365D)/Github_Projects/ECC/openlxp-xia-1/app/enviroment_name/bin/python
from django.core import management

if __name__ == "__main__":
    management.execute_from_command_line()
