# Generated by Django 3.2.13 on 2023-07-14 16:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0004_alter_xiaconfiguration_xss_api'),
    ]

    operations = [
        migrations.AlterField(
            model_name='xisconfiguration',
            name='xis_api_key',
            field=models.CharField(help_text='Enter the XIS API Key', max_length=128),
        ),
    ]
