# Generated by Django 3.2.18 on 2025-02-19 17:19

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0006_auto_20230907_1642'),
    ]

    operations = [
        migrations.AddField(
            model_name='xiaconfiguration',
            name='key_fields',
            field=models.TextField(default="['p2881_course_profile.Course_ID', 'p2881_course_profile.CourseProviderName']", help_text='Enter list of field names to create metadata key'),
        ),
        migrations.AlterField(
            model_name='metadataledger',
            name='source_metadata',
            field=models.JSONField(blank=True, validators=[django.core.validators.RegexValidator(message='The Wrong Format Entered', regex='^([\\x09\\x0A\\x0D\\x20-\\x7E]|[\\xC2-\\xDF]|[\\xE0\\xA0-\\xBF]|[\\xE1-\\xEC\\xEE\\xEF]{2}|[\\xED\\x80-\\x9F]|[\\xF0\\x90-\\xBF]{2}|[\\xF1-\\xF3]{3}|[\\xF4\\x80-\\x8F]{2})*$')]),
        ),
        migrations.AlterField(
            model_name='supplementalledger',
            name='supplemental_metadata',
            field=models.JSONField(blank=True, validators=[django.core.validators.RegexValidator(message='The Wrong Format Entered', regex='^([\\x09\\x0A\\x0D\\x20-\\x7E]|[\\xC2-\\xDF]|[\\xE0\\xA0-\\xBF]|[\\xE1-\\xEC\\xEE\\xEF]{2}|[\\xED\\x80-\\x9F]|[\\xF0\\x90-\\xBF]{2}|[\\xF1-\\xF3]{3}|[\\xF4\\x80-\\x8F]{2})*$')]),
        ),
    ]
